@@ -334,9 +334,10 @@ class Estimator:
                 batch = self._configure_tensor(loader, batch)
                 batch, prediction = self.network.run_step(batch)
                 self._run_traces_on_batch_end(batch, prediction, traces=traces)
-                if (self.system.batch_idx == self.system.max_train_steps_per_epoch and self.system.mode == "train") or (
-                        self.system.batch_idx == self.system.max_eval_steps_per_epoch and self.system.mode == "eval"):
-                    break
+                if isinstance(loader, DataLoader) and (
+                    (self.system.batch_idx == self.system.max_train_steps_per_epoch and self.system.mode == "train") or
+                    (self.system.batch_idx == self.system.max_eval_steps_per_epoch and self.system.mode == "eval")):
+                    raise StopIteration
                 with Suppressor():
                     batch = next(iterator)
             except StopIteration:
