@@ -17,7 +17,7 @@ import random
 import time
 import warnings
 from copy import deepcopy
-from typing import Any, Dict, List, MutableMapping, Optional, Set, TypeVar, Union
+from typing import Any, Callable, Dict, List, MutableMapping, Optional, Set, TypeVar, Union
 
 import numpy as np
 import tensorflow as tf
@@ -50,6 +50,7 @@ class Pipeline:
         drop_last: Whether to drop the last batch if the last batch is incomplete.
         pad_value: The padding value if batch padding is needed. None indicates that no padding is needed. NOTE: This
             argument is only applicable when using a FastEstimator Dataset.
+        collate_fn: Function to merge data into one batch with input being list of elements.
     """
     ops: List[Union[NumpyOp, Scheduler[NumpyOp]]]
 
@@ -61,13 +62,15 @@ class Pipeline:
                  ops: Union[None, NumpyOp, Scheduler[NumpyOp], List[Union[NumpyOp, Scheduler[NumpyOp]]]] = None,
                  num_process: Optional[int] = None,
                  drop_last: bool = False,
-                 pad_value: Optional[Union[int, float]] = None):
+                 pad_value: Optional[Union[int, float]] = None,
+                 collate_fn: Optional[Callable] = None):
         self.data = {x: y for (x, y) in zip(["train", "eval", "test"], [train_data, eval_data, test_data]) if y}
         self.batch_size = batch_size
         self.ops = to_list(ops)
         self.num_process = num_process if num_process is not None else os.cpu_count() if os.name != 'nt' else 0
         self.drop_last = drop_last
         self.pad_value = pad_value
+        self.collate_fn = collate_fn
         self._verify_inputs(**{k: v for k, v in locals().items() if k != 'self'})
 
     def _verify_inputs(self, **kwargs) -> None:
@@ -262,15 +265,12 @@ class Pipeline:
             # batch dataset
             if isinstance(data, BatchDataset):
                 data.pad_value = self.pad_value
-            else:
-                assert batch_size is not None, "batch_size should not be None"
             # shuffle
             if shuffle is None:
-                shuffle = mode == "train"
+                shuffle = mode == "train" and batch_size is not None
             # collate_fn
-            if self.pad_value is None or isinstance(data, BatchDataset):
-                collate_fn = None
-            else:
+            collate_fn = self.collate_fn
+            if collate_fn is None and self.pad_value is not None:
                 collate_fn = self._pad_batch_collate
             op_dataset = OpDataset(data, get_current_items(self.ops, mode, epoch), mode)
             data = DataLoader(op_dataset,
